from dennis.tools import VariableTokenizer

from nose.tools import eq_


def test_python_tokenizing():
    vartok = VariableTokenizer(['python'])
    for string, expected in [
        ('Hello %s', ['Hello ', '%s', '']),
        ('Hello %(username)s', ['Hello ', '%(username)s', '']),
        ('Hello %(user)s%(name)s', ['Hello ', '%(user)s', '', '%(name)s', '']),
        ('Hello {username}', ['Hello ', '{username}', '']),
        ('Hello {user}{name}', ['Hello ', '{user}', '', '{name}', '']),
        ('Products and Services', ['Products and Services']),
        ]:
        eq_(vartok.tokenize(string), expected)



import os
import sys

from dennis import __version__
from dennis.linter import Linter, format_with_errors
from dennis.tools import BetterArgumentParser, Terminal, get_types
from dennis.translator import Translator, get_available_pipeline_parts


USAGE = '%prog [options] [command] [command-options]'
VERSION = '%prog ' + __version__

# blessings.Terminal and our mock Terminal don't maintain any state
# so we can just make it global
TERMINAL = Terminal()


def build_parser(usage, **kwargs):
    """Builds an OptionParser with the specified kwargs."""
    return BetterArgumentParser(usage=usage, version=VERSION, **kwargs)


def err(s):
    """Prints a single-line string to stderr."""
    sys.stderr.write('Error: ' + s + '\n')


def format_types():
    return (
        '\nAvailable Types:\n' +
        '\n'.join(
            ['  {name:10}  {desc}'.format(name=name, desc=desc)
             for name, desc in get_types()])
    )


def format_pipeline_parts():
    parts = sorted(get_available_pipeline_parts().items())
    return (
        '\nAvailable Pipeline Parts:\n' +
        '\n'.join(
            ['  {name:10}  {desc}'.format(name=name, desc=cls.__doc__)
             for name, cls in parts])
    )


def print_lint_error(vartok, lint_error):
    """Prints a LintError to stdout

    :arg vartok: VariableTokenizer instance
    :arg lint_error: a LintError to print

    Prints it to stdout. It also colorizes it using blessings
    if blessings is available.
    """
    if lint_error.invalid:
        print u'{label}: {tokens}'.format(
            label=TERMINAL.bold_red('Error: invalid tokens'),
            tokens=', '.join(lint_error.invalid))

    if lint_error.missing:
        print u'{label}: {tokens}'.format(
            label=TERMINAL.bold_yellow('Warning: missing tokens'),
            tokens=u', '.join(lint_error.missing))

    name = TERMINAL.yellow('msgid')
    print u'{0}: "{1}"'.format(name, lint_error.msgid)

    if lint_error.index is not None:
        # Print the plural
        name = TERMINAL.yellow('msgid_plural')
        print u'{0}: "{1}"'.format(name, lint_error.msgid_text)

    # Print the translated string with token errors
    if lint_error.index is not None:
        name = 'msgstr[{index}]'.format(index=lint_error.index)
    else:
        name = 'msgstr'
    print u'{0}: "{1}"'.format(
        TERMINAL.yellow(name),
        format_with_errors(
            TERMINAL, vartok, lint_error.msgstr_text, lint_error.msgid_tokens))

    print ''


def lint_cmd(scriptname, command, argv):
    """Lints a .po file or directory of files."""
    if not '--quiet' in argv:
        print '%s version %s' % (scriptname, __version__)

    parser = build_parser(
        'usage: %prog lint [ FILE | DIR ]',
        description='Lints a .po file for mismatched Python string '
        'formatting tokens.',
        sections=[
            (format_types(), True),
        ])
    parser.add_option(
        '-t', '--types',
        dest='types',
        help='Comma-separated list of variable types. See Available Types.',
        metavar='TYPES',
        default='python')
    parser.add_option(
        '-q', '--quiet',
        action='store_true',
        dest='quiet',
        help='quiet all output')

    (options, args) = parser.parse_args(argv)

    if not args:
        parser.print_help()
        return 1

    linter = Linter(options.types.split(','))

    if os.path.isdir(args[0]):
        po_files = []
        for root, dirs, files in os.walk(args[0]):
            po_files.extend(
                [os.path.join(root, fn) for fn in files
                 if fn.endswith('.po')])

    else:
        po_files = [args[0]]

    files_to_errors = {}
    total_error_count = 0
    total_warning_count = 0
    total_files_with_errors = 0

    for fn in po_files:
        if not fn.endswith('.po'):
            continue

        fn = os.path.abspath(fn)

        results = linter.verify_file(fn)

        # This is the total number of strings examined.
        count = len(results)

        # Extract all the problematic LintItems--they have non-empty
        # missing or invalid lists.
        problem_results = [r for r in results if r.missing or r.invalid]

        # We don't want to print output for files that are fine, so we
        # update the bookkeeping and move on.
        if not problem_results:
            files_to_errors[fn] = (0, 0)
            continue

        if not options.quiet:
            print TERMINAL.bold_green('>>> Working on: {fn}'.format(fn=fn))

        error_count = 0
        warning_count = 0
        for result in problem_results:
            if not result:
                continue

            if result.invalid:
                total_error_count += 1
                error_count += 1
            if result.missing:
                total_warning_count += 1
                warning_count += 1

            if not options.quiet:
                print_lint_error(linter.vartok, result)

        files_to_errors[fn] = (error_count, warning_count)

        if error_count > 0:
            total_files_with_errors += 1

        if not options.quiet:
            print (
                'Total: {count:5}  Warnings: {warnings:5}  Errors: {errors:5}'
                .format(count=count, warnings=warning_count, errors=error_count))
            print ''

    if len(po_files) > 1 and not options.quiet:
        print 'Final Tally:'
        print ''

        print 'Number of files examined:          {count:5}'.format(
            count=len(po_files))
        print 'Total number of files with errors: {count:5}'.format(
            count=total_files_with_errors)
        print 'Total number of warnings:          {count:5}'.format(
            count=total_warning_count)
        print 'Total number of errors:            {count:5}'.format(
            count=total_error_count)
        print ''

        file_counts = [
            (counts[0], counts[1], fn.split(os.sep)[-3], fn.split(os.sep)[-1])
            for (fn, counts) in files_to_errors.items()]

        print 'Warnings  Errors  Filename'
        file_counts = reversed(sorted(file_counts))
        for error_count, warning_count, locale, fn in file_counts:
            if not error_count and not warning_count:
                continue

            print ' {warnings:5}     {errors:5}  {locale} ({fn})'.format(
                warnings=warning_count, errors=error_count, fn=fn,
                locale=locale)

    # Return 0 if everything was fine or 1 if there were errors.
    return 1 if total_error_count else 0


def translate_cmd(scriptname, command, argv):
    """Translate a single string or .po file of strings."""
    print '%s version %s' % (scriptname, __version__)

    parser = build_parser(
        'usage: %prog tramslate '
        '[-s STRING <STRING> ... | FILENAME <FILENAME> ...]',
        description='Translates a string or a .po file into Pirate.',
        epilog='Note: Translating files is done in-place replacing '
        'the original file.',
        sections=[
            (format_types(), True),
            (format_pipeline_parts(), True),
        ])
    # FIXME: move printing of available types to epilog. also rename
    # types to something more accurate like "variable formats".
    parser.add_option(
        '-t', '--types',
        dest='types',
        help='Comma-separated list of variable types. See Available Types.',
        metavar='TYPES',
        default='python')
    parser.add_option(
        '-p', '--pipeline',
        dest='pipeline',
        help='Translate pipeline. See Available Pipeline Parts.',
        metavar='PIPELINE',
        default='html,pirate')
    parser.add_option(
        '-s', '--string',
        action='store_true',
        dest='strings',
        help='translates specified string args')

    (options, args) = parser.parse_args(argv)

    if not args:
        parser.print_help()
        return 1

    translator = Translator(
        options.types.split(','), options.pipeline.split(','))

    if options.strings:
        for arg in args:
            print translator.translate_string(arg)
        return 0

    for arg in args:
        translator.translate_file(arg)
    return 0


def get_handlers():
    handlers = [(name.replace('_cmd', ''), fun, fun.__doc__)
                for name, fun in globals().items()
                if name.endswith('_cmd')]
    return handlers


def print_help(scriptname):
    print '%s version %s' % (scriptname, __version__)

    handlers = get_handlers()

    parser = build_parser("%prog [command]")
    parser.print_help()
    print ''
    print 'Commands:'
    for command_str, _, command_help in handlers:
        print '  {cmd:10}  {hlp}'.format(cmd=command_str, hlp=command_help)


def cmdline_handler(scriptname, argv):
    handlers = get_handlers()

    if not argv or argv[0] in ('-h', '--help'):
        print_help(scriptname)
        return 0

    if '--version' in argv:
        # We've already printed the version, so we can just exit.
        return 0

    command = argv.pop(0)
    for (cmd, fun, hlp) in handlers:
        if cmd == command:
            return fun(scriptname, command, argv)

    err('Command "{0}" does not exist.'.format(command))
    print_help(scriptname)

    return 1



def build_po_string(data):
    return (
        '#, fuzzy\n'
        'msgid ""\n'
        'msgstr ""\n'
        '"Project-Id-Version: foo\\n"\n'
        '"POT-Creation-Date: 2013-06-05 14:16-0700\\n"\n'
        '"PO-Revision-Date: 2010-04-26 18:00-0700\\n"\n'
        '"Last-Translator: Automatically generated\\n"\n'
        '"Language-Team: English\\n"\n'
        '"Language: \\n"\n'
        '"MIME-Version: 1.0\\n"\n'
        '"Content-Type: text/plain; charset=UTF-8\\n"\n'
        '"Content-Transfer-Encoding: 8bit\\n"\n'
        '"X-Generator: Translate Toolkit 1.6.0\\n"\n\n'
        + data)



try:
    from blessings import Terminal
except ImportError:
    class MockBlessedThing(object):
        def __call__(self, s):
            return s

        def __str__(self):
            return ''

        def __repr__(self):
            return ''

        def __unicode__(self):
            return ''

    class Terminal(object):
        def __getattr__(self, attr, default=None):
            return MockBlessedThing()

import optparse
import re


class PythonVarType(object):
    """Python %(foo)s and {foo} syntax"""
    name = 'python'
    regexp = (
        r'(?:%(?:[(]\S+?[)])?[#0+-]?[\.\d\*]*[hlL]?[diouxXeEfFgGcrs%])'
        r'|'
        r'(?:\{\S+?\})'
    )


VAR_TYPES = dict(
    (var_class.name, (var_class, var_class.__doc__))
    for name, var_class in globals().items()
    if name.endswith('VarType')
)


class UnknownVarType(Exception):
    pass


def get_types():
    """Return name and description for all types"""
    return [(name, data[1]) for name, data in sorted(VAR_TYPES.items())]


class VariableTokenizer(object):
    def __init__(self, var_types=None):
        """
        :arg var_types: List of types.

            If None, creates a VariableTokenizer that tokenizes on all
            types of variables. Otherwise just recognizes the listed
            types.

        """
        if var_types is None:
            var_types = VAR_TYPES.keys()

        # Convert names to classes
        self.var_types = []

        for v in var_types:
            try:
                self.var_types.append(VAR_TYPES[v][0])
            except KeyError:
                raise UnknownVarType(
                    '{0} is not a known variable type'.format(v))

        # Generate variable regexp
        self.var_re = re.compile(
            r'(' +
            '|'.join([vt.regexp for vt in self.var_types]) +
            r')'
        )

    def tokenize(self, text):
        """Breaks s into strings and Python formatting tokens

        This preserves whitespace.

        :arg text: the string to tokenize

        :returns: list of tokens---every even one is a Python formatting
            token

        """
        return self.var_re.split(text)

    def extract_tokens(self, text):
        """Returns sorted tuple of tokens in the text"""
        try:
            tokens = [token for token in self.var_re.findall(text)]
            return tuple(sorted(tokens))
        except TypeError:
            print 'TYPEERROR', repr(text)

    def is_token(self, text):
        return self.var_re.match(text) is not None


class BetterArgumentParser(optparse.OptionParser):
    """OptionParser that allows for additional help sections

    OptionParser can take a description and an epilog, but it
    textwraps them which destroys all formatting. This allows us to
    have additional sections after the epilog which can maintain
    formatting.

    When creating the parser, pass in a ``sections`` kw argument with
    a list of tuples of the form ``(text, boolean)``. The text is the
    section text. The boolean indicates whether or not to textwrap the
    text.

    Example::

        BetterArgumentParser(usage='usage: %prog blah blah', version='1.0',
            sections=[
                ('List\nof\nthings', False),  # Maintains format
                ('List\nof\nthings', True),   # Textwrapped
            ])

    """
    def __init__(self, *args, **kw):
        if 'sections' in kw:
            self.sections = kw.pop('sections')
        else:
            self.sections = []
        optparse.OptionParser.__init__(self, *args, **kw)

    def format_help(self, formatter=None):
        help_text = optparse.OptionParser.format_help(self, formatter)
        for (section, raw) in self.sections:
            if raw:
                help_text += section
            else:
                help_text += self._format_text(section)
            help_text += '\n'

        return help_text



import HTMLParser
import polib
import re
import string

from dennis.tools import VariableTokenizer


DEBUG = False


def debug(*args):
    if DEBUG:
        print ' '.join([str(arg) for arg in args])


class Token(object):
    def __init__(self, s, type='text', mutable=True):
        self.s = s
        self.type = type
        self.mutable = mutable

    def __str__(self):
        return self.s.encode('utf-8')

    def __unicode__(self):
        return self.s

    def __repr__(self):
        return '<{0} {1}>'.format(self.type, self.s.encode('utf-8'))

    def __eq__(self, token):
        return (
            self.s == token.s
            and self.mutable == token.mutable
            and self.type == token.type)


class Transform(object):
    name = ''

    def transform(self, vartok, token_stream):
        """Takes a token stream and returns a token stream

        :arg vartok: the variable tokenizer to use for tokenizing
            on variable tokens
        :arg token_stream: the input token stream this transform
            is operating on

        :return: iterable of transformed tokens

        """
        raise NotImplemented


class PirateTransform(Transform):
    """Translates text into Pirate!"""
    name = 'pirate'

    def __init__(self):
        pass

    def transform(self, vartok, token_stream):
        new_tokens = []

        for token in token_stream:
            if not token.mutable:
                new_tokens.append(token)
                continue

            out = u''
            for i, part in enumerate(vartok.tokenize(token.s)):
                if i % 2 == 0:
                    out += self.pirate_transform(part)
                else:
                    out += part

            # Add color which causes every string to be longer.
            s, ending = self.split_ending(out)
            out = s + u' ' + self.COLOR[len(out) % len(self.COLOR)] + ending

            # This guarantees that every string has at least one
            # unicode charater
            if '!' not in out:
                out = out + u'!'

            # Replace all ! with related unicode character.
            out = out.replace(u'!', u'\u2757')

            new_tokens.append(Token(out))

        return new_tokens

    COLOR = [
        u'arr!',
        u'arrRRr!',
        u'arrRRRrrr!',
        u'matey!',
        u'me mateys!',
        u'ahoy!',
        u'aye!',
        u'ye scalleywag!',
        u'cap\'n!',
        u'yo-ho-ho!',
        u'shiver me timbers!',
        u'ye landlubbers!',
        u'prepare to be boarded!',
        ]

    def wc(self, c):
        return c == '\'' or c in string.letters

    def nwc(self, c):
        return not self.wc(c)

    def is_whitespace(self, s):
        return re.match('^\\s*$', s) is not None

    # List of transform rules. The tuples have:
    #
    # * in word
    # * not in word
    # * match string
    # * word character
    # * not word character
    # * replacement string
    #
    # See the transform code for details.
    TRANSFORM = (
        # INW?, NIW?, match, WC?, NW?, replacement
        # Anti-replacements: need these so that we make sure these words
        # don't get screwed up by later rules.
        (False, True, 'need', False, True, 'need'),
        (False, True, 'Need', False, True, 'Need'),

        # Replace entire words
        (False, True, 'add-on', False, True, 'bilge rat'),
        (False, True, 'add-ons', False, True, 'bilge rats'),
        (False, True, 'are', False, True, 'bee'),
        (False, True, 'browser', False, True, 'corsairr'),
        (False, True, 'for', False, True, 'fer'),
        (False, True, 'Hi', False, True, 'H\'ello'),
        (False, True, 'my', False, True, 'me'),
        (False, True, 'no', False, True, 'nay'),
        (False, True, 'of', False, True, 'o\''),
        (False, True, 'over', False, True, 'o\'err'),
        (False, True, 'plugin', False, True, 'mug o\' grog'),
        (False, True, 'plugins', False, True, 'mugs o\' grog'),
        (False, True, 'program', False, True, 'Jolly Rogerr'),
        (False, True, 'the', False, True, 'th\''),
        (False, True, 'there', False, True, 'tharr'),
        (False, True, 'want', False, True, 'wants'),
        (False, True, 'where', False, True, '\'erre'),
        (False, True, 'with', False, True, 'wit\''),
        (False, True, 'yes', False, True, 'aye'),
        (False, True, 'you', False, True, 'ye\''),
        (False, True, 'You', False, True, 'Ye\''),
        (False, True, 'your', False, True, 'yerr'),
        (False, True, 'Your', False, True, 'Yerr'),

        # Prefixes
        (False, True, 'hel', True, False, '\'el'),
        (False, True, 'Hel', True, False, '\'el'),

        # Mid-word
        (True, False, 'er', True, False, 'arr'),

        # Suffixes
        (True, False, 'a', False, True, 'ar'),
        (True, False, 'ed', False, True, '\'d'),
        (True, False, 'ing', False, True, 'in\''),
        (True, False, 'ort', False, True, 'arrt'),
        (True, False, 'r', False, True, 'rr'),
        (True, False, 'w', False, True, 'ww'),
    )

    def split_ending(self, s):
        ending = []
        while s:
            if s[-1] in '.,":;?!':
                ending.insert(0, s[-1])
                s = s[:-1]
            else:
                return s, u''.join(ending)

        return u'', ''.join(ending)

    def pirate_transform(self, s):
        """Transforms a token into Pirate.

        :arg s: The token to transform.

        :returns: Piratized token

        """
        out = []

        in_word = False  # in a word?

        # TODO: This is awful--better to do a real lexer
        while s:
            if s.startswith(('.', '!', '?')):
                in_word = False
                out.append(s[0])
                s = s[1:]
                continue

            debug(s, in_word)

            for mem in self.TRANSFORM:
                # Match inside a word? (Not a prefix.)
                if in_word and not mem[0]:
                    debug(mem, 'not in word')
                    continue

                # Not match inside a word? (Prefix.)
                if not in_word and not mem[1]:
                    debug(mem, 'in word')
                    continue

                if not s.startswith(mem[2]):
                    debug(mem, 'not match')
                    continue

                # Check the character after the match to see if it's a
                # word character and whether this match covers that.
                try:
                     # WC: word character
                    if mem[3] and not self.wc(s[len(mem[2])]):
                        debug(mem, 'not wc')
                        continue
                except IndexError:
                    # We don't count EOS as a word character.
                    if mem[3]:
                        debug(mem, 'not wc')
                        continue

                # Check the character after the match to see if it's not a
                # word character and whether this match covers that.
                try:
                    # NW: not word character
                    if mem[4] and not self.nwc(s[len(mem[2])]):
                        debug(mem, 'wc')
                        continue
                except IndexError:
                    # We count EOS as a non-word character.
                    if not mem[4]:
                        debug(mem, 'wc')
                        continue

                out.append(mem[5])
                s = s[len(mem[2]):]
                in_word = True
                break

            else:
                in_word = self.wc(s[0])
                out.append(s[0])
                s = s[1:]

        return u''.join(out)


class HTMLExtractorTransform(HTMLParser.HTMLParser, Transform):
    """Tokenizes HTML bits so only text is translated."""
    name = 'html'

    def __init__(self):
        HTMLParser.HTMLParser.__init__(self)
        self.new_tokens = []

    def transform(self, vartok, token_stream):
        out = []

        for token in token_stream:
            if not token.mutable:
                out.append(token)
                continue

            if token.s:
                self.feed(token.s)
                out.extend(self.new_tokens)
                self.new_tokens = []
            else:
                out.append(token)

        return out

    def handle_starttag(self, tag, attrs, closed=False):
        # We want to translate alt and title values, but that's
        # it. So this gets a little goofy looking token-wise.

        s = '<' + tag
        for name, val in attrs:
            s += ' '
            s += name
            s += '="'

            if name in ['alt', 'title']:
                self.new_tokens.append(Token(s, 'html', False))
                self.new_tokens.append(Token(val))
                s = ''
            else:
                s += val
            s += '"'
        if closed:
            s += ' /'
        s += '>'

        if s:
            self.new_tokens.append(Token(s, 'html', False))

    def handle_startendtag(self, tag, attrs):
        self.handle_starttag(tag, attrs, closed=True)

    def handle_endtag(self, tag):
        self.new_tokens.append(Token('</' + tag + '>', 'html', False))

    def handle_data(self, data):
        self.new_tokens.append(Token(data))

    def handle_charref(self, name):
        self.new_tokens.append(Token('&#' + name + ';', 'html', False))

    def handle_entityref(self, name):
        self.new_tokens.append(Token('&' + name + ';', 'html', False))


def get_available_pipeline_parts():
    pipeline_parts = {}

    for name, thing in globals().items():
        if (name.endswith('Transform')
            and issubclass(thing, Transform)
            and thing.name):
            pipeline_parts[thing.name] = thing

    return pipeline_parts


class InvalidPipeline(Exception):
    pass


def convert_pipeline(pipeline_spec):
    pipeline_parts = get_available_pipeline_parts()

    try:
        pipeline = [pipeline_parts[part] for part in pipeline_spec]
    except KeyError:
        raise InvalidPipeline(pipeline_spec)

    return pipeline


class Translator(object):
    def __init__(self, variable_formats, pipeline_spec):
        self.vartok = VariableTokenizer(variable_formats)
        self.pipeline_spec = pipeline_spec
        self.pipeline = convert_pipeline(self.pipeline_spec)

    def translate_string(self, s):
        # Create the initial token
        tokens = [Token(s)]

        # Pass the token list through the pipeline
        for part_class in self.pipeline:
            part = part_class()
            tokens = part.transform(self.vartok, tokens)

        # Join all the bits together
        return u''.join([token.s for token in tokens])

    def translate_file(self, fname):
        po = polib.pofile(fname)

        # FIXME - This might be a bit goofy
        po.metadata['Language'] = ",".join(self.pipeline_spec)
        po.metadata['Plural-Forms'] = 'nplurals=2; plural= n != 1'
        po.metadata['Content-Type'] = 'text/plain; charset=UTF-8'
        count = 0
        for entry in po:
            if entry.msgid_plural:
                entry.msgstr_plural['0'] = self.translate_string(
                    entry.msgid)
                entry.msgstr_plural['1'] = self.translate_string(
                    entry.msgid_plural)
            else:
                entry.msgstr = self.translate_string(entry.msgid)

            if 'fuzzy' in entry.flags:
                entry.flags.remove('fuzzy')  # clear the fuzzy flag
            count += 1
        print 'Munged %d messages in %s' % (count, fname)
        po.save()



from unittest import TestCase

from nose.tools import eq_

from dennis.tools import VariableTokenizer
from dennis.translator import (
    HTMLExtractorTransform, PirateTransform, Token, Translator)


class PirateTest(TestCase):
    vartok = VariableTokenizer(['python'])

    def test_basic(self):
        data = [
            (u'Hello',
             u'\'ello ahoy\u2757'),

            (u'Hello %(username)s',
             u'\'ello %(username)s ahoy\u2757'),

            (u'Hello %s',
             u'\'ello %s cap\'n\u2757'),

            (u'Hello {user}{name}',
             u'\'ello {user}{name} ahoy\u2757'),

            (u'Products and Services',
             u'Products and Sarrvices yo-ho-ho\u2757'),

            (u'Get community support',
             u'Get community supparrt yo-ho-ho\u2757'),

            (u'Your input helps make Mozilla better',
             u'Yerr input \'elps make Mozillar betterr prepare to '
             u'be boarded\u2757'),

            (u'Super browsing',
             u'Superr browsin\' arrRRRrrr\u2757'),
        ]

        for text, expected in data:
            pt = PirateTransform()
            output = pt.transform(self.vartok, [Token(text)])
            output = u''.join([token.s for token in output])

            eq_(output, expected)


class HTMLExtractorTest(TestCase):
    vartok = VariableTokenizer(['python'])

    def test_basic(self):
        htmle = HTMLExtractorTransform()
        output = htmle.transform(self.vartok, [Token('')])
        eq_(output, [Token(u'', 'text', True)])

        output = htmle.transform(self.vartok, [Token('<b>hi</b>')])
        eq_(output,
            [
                Token(u'<b>', 'html', False),
                Token(u'hi', 'text', True),
                Token(u'</b>', 'html', False),
            ]
        )

    def test_alt_title(self):
        htmle = HTMLExtractorTransform()
        output = htmle.transform(self.vartok, [
                Token('<img alt="foo">')])
        eq_(output,
            [
                Token(u'<img alt="', 'html', False),
                Token(u'foo', 'text', True),
                Token(u'">', 'html', False),
            ]
        )

        output = htmle.transform(self.vartok, [
                Token('<img title="foo">')])
        eq_(output,
            [
                Token(u'<img title="', 'html', False),
                Token(u'foo', 'text', True),
                Token(u'">', 'html', False),
            ]
        )


class TranslatorTest(TestCase):
    def test_pirate_translate(self):
        trans = Translator(['python'], ['pirate'])
        eq_(trans.translate_string(u'hello'),
            u'\'ello ahoy\u2757')

        # Note, this doesn't work right because it's not accounting
        # for html. But it's correct for this test.
        eq_(trans.translate_string(u'<b>hello</b>'),
            u'<b>\'ello</b> prepare to be boarded\u2757')

    def test_html_pirate_translate(self):
        trans = Translator(['python'], ['html', 'pirate'])
        eq_(trans.translate_string(u'<b>hello</b>'),
            u'<b>\'ello ahoy\u2757</b>')



#######################################################################
# This file is part of dennis.
#
# Copyright (C) 2013 Will Kahn-Greene
# Licensed under the Simplified BSD License. See LICENSE for full
# license.
#######################################################################

from setuptools import setup, find_packages
import re
import os


READMEFILE = 'README.rst'
VERSIONFILE = os.path.join('dennis', '__init__.py')
VSRE = r"""^__version__ = ['"]([^'"]*)['"]"""


def get_version():
    version_file = open(VERSIONFILE, 'rt').read()
    return re.search(VSRE, version_file, re.M).group(1)


setup(
    name='dennis',
    version=get_version(),
    description='l10n and PO file helper tools',
    long_description=open(READMEFILE).read(),
    license='Simplified BSD License',
    author='Will Kahn-Greene',
    author_email='willg@bluesock.org',
    keywords='l10n localization po',
    url='http://github.com/willkg/dennis',
    zip_safe=True,
    packages=find_packages(),
    include_package_data=True,
    scripts=['bin/dennis-cmd'],
    install_requires=[
        'polib',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)



import polib

from dennis.tools import VariableTokenizer


def _compare_lists(list_a, list_b):
    """Compares contents of two lists

    This returns two lists:

    * list of tokens in list_a missing from list_b

    * list of tokens in list_b missing from list_a

    :returns: tuple of (list of tokens in list_a not in list_b, list
        of tokens in list_b not in list_a)

    """
    return (
        [token for token in list_a if token not in list_b],
        [token for token in list_b if token not in list_a]
    )


class LintItem(object):
    """Holds all the data involved in a lint item"""
    def __init__(self, poentry, msgid, msgid_text, msgid_tokens,
                 msgstr_text, msgstr_tokens, index):
        self.poentry = poentry

        self.msgid = msgid
        self.msgid_text = msgid_text
        self.msgid_tokens = msgid_tokens
        self.msgstr_text = msgstr_text
        self.msgstr_tokens = msgstr_tokens
        self.index = index

        # If the str_text is empty, there's no translation which we
        # consider "fine".
        if not msgstr_text.strip():
            self.missing, self.invalid = [], []
        else:
            self.missing, self.invalid = _compare_lists(
                msgid_tokens, msgstr_tokens)


class Linter(object):
    def __init__(self, var_types):
        # FIXME - this is a horrible name
        self.vartok = VariableTokenizer(var_types)

    def verify_file(self, filename_or_string):
        """Verifies strings in file.

        :arg filename_or_string: filename to verify or the contents of
            a pofile as a string

        :returns: for each string in the pofile, this returns a None
            if there were no issues or a LintError if there were
            issues

        """
        po = polib.pofile(filename_or_string)

        results = []

        for entry in po:
            if not entry.msgid_plural:
                if not entry.msgid and entry.msgstr:
                    continue
                id_tokens = self.vartok.extract_tokens(entry.msgid)
                str_tokens = self.vartok.extract_tokens(entry.msgstr)

                results.append(
                    LintItem(entry, entry.msgid, entry.msgid,
                             id_tokens, entry.msgstr,
                             str_tokens, None))

            else:
                for key in sorted(entry.msgstr_plural.keys()):
                    if key == '0':
                        # This is the 1 case.
                        text = entry.msgid
                    else:
                        text = entry.msgid_plural
                    id_tokens = self.vartok.extract_tokens(text)

                    str_tokens = self.vartok.extract_tokens(
                        entry.msgstr_plural[key])
                    results.append(
                        LintItem(entry, entry.msgid, text, id_tokens,
                                 entry.msgstr_plural[key],
                                 str_tokens, key))

        return results


def format_with_errors(terminal, vartok, text, available_tokens):
    """Turns invalid tokens in the text red for output

    :arg terminal: a blessings Terminal or mock Terminal
    :arg vartok: the variable tokenizer to use
    :arg text: the text holding the tokens
    :arg available_tokens: valid tokens that should be in the
        text---anything else is an invalid token and will be turned
        bold red.

    :returns: the text with invalid tokens bold red

    """
    output = []
    for token in vartok.tokenize(text):
        if vartok.is_token(token) and token not in available_tokens:
            output.append(terminal.bold_red(token))
        else:
            output.append(token)
    return u''.join(output)



from unittest import TestCase

from nose.tools import eq_

from dennis.linter import _compare_lists, Linter
from dennis.tests import build_po_string


def test_compare_lists():
    tests = [
        ([], [], ([], [])),
        ([1, 2, 3], [3, 4, 5], ([1, 2], [4, 5])),
    ]

    for list_a, list_b, expected in tests:
        eq_(_compare_lists(list_a, list_b), expected)


class LinterTests(TestCase):
    def test_fine(self):
        po_string = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo"\n'
            'msgstr "Oof"\n')

        linter = Linter(['python'])

        results = linter.verify_file(po_string)
        eq_(len(results), 1)
        eq_(results[0].missing, [])
        eq_(results[0].invalid, [])

        po_string = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo: {foo}"\n'
            'msgstr "Oof: {foo}"\n')

        linter = Linter(['python'])

        results = linter.verify_file(po_string)
        eq_(len(results), 1)
        eq_(results[0].missing, [])
        eq_(results[0].invalid, [])

    def test_missing(self):
        data = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo: {foo}"\n'
            'msgstr "Oof"\n')

        linter = Linter(['python'])

        results = linter.verify_file(data)
        eq_(len(results), 1)
        eq_(results[0].missing, [u'{foo}'])
        eq_(results[0].invalid, [])

        data = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo: {foo} {bar} {baz}"\n'
            'msgstr "Oof: {foo}"\n')

        linter = Linter(['python'])

        results = linter.verify_file(data)
        eq_(len(results), 1)
        eq_(results[0].missing, [u'{bar}', u'{baz}'])
        eq_(results[0].invalid, [])

    def test_invalid(self):
        data = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo"\n'
            'msgstr "Oof: {foo}"\n')

        linter = Linter(['python'])

        results = linter.verify_file(data)
        eq_(len(results), 1)
        eq_(results[0].missing, [])
        eq_(results[0].invalid, [u'{foo}'])

        data = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo: {foo}"\n'
            'msgstr "Oof: {foo} {bar} {baz}"\n')

        linter = Linter(['python'])

        results = linter.verify_file(data)
        eq_(len(results), 1)
        eq_(results[0].missing, [])
        eq_(results[0].invalid, [u'{bar}', u'{baz}'])

    def test_complex(self):
        data = build_po_string(
            '#: foo/foo.py:5\n'
            'msgid "Foo: {bar} {baz}"\n'
            'msgstr "Oof: {foo} {bar}"\n')

        linter = Linter(['python'])

        results = linter.verify_file(data)
        eq_(len(results), 1)
        eq_(results[0].missing, [u'{baz}'])
        eq_(results[0].invalid, [u'{foo}'])
